;;; Verum-Dezyne --- An IDE for Dezyne
;;;
;;; Copyright © 2020 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
;;;
;;; This file is part of Verum-Dezyne.
;;;
;;; Verum-Dezyne is property of Verum Software Tools BV <support@verum.com>.
;;; All rights reserved.

;;
;;; Commentary:
;;;
;;; Code:

(define-module (ide commands hello)
  #:use-module (ice-9 getopt-long)
  #:use-module (ide commander)
  #:use-module (ide commands daemon)
  #:use-module (ide config)
  #:use-module (ide daemon-config)
  #:use-module (ide command-line)
  #:use-module (ide util))

(define (parse-opts args)
  (let* ((option-spec
          '((force (single-char #\f))
            (help (single-char #\h))
            (import (single-char #\I) (value #t))
            (model (single-char #\m) (value #t))
            (verbose (single-char #\v))))
	 (options (getopt-long args option-spec))
	 (help? (option-ref options 'help #f))
	 (files (option-ref options '() '()))
         (usage? (and (not help?) (pair? files))))
    (when (or help? usage?)
      (let ((port (if usage? (current-error-port) (current-output-port))))
        (format port "\
Usage: ide hello [OPTION]...
Start Dezyne session, check daemon.

  -f, --force            remove stale PID-file
  -h, --help             display this help and exit
  -v, --verbose          be verbose; show connections with daemon
")
        (exit (or (and usage? EXIT_OTHER_FAILURE) EXIT_SUCCESS))))
    options))

(define (main args)
  (daemon-config-set!)
  (let* ((options (parse-opts args))
         (debug? (ide:command-line:get 'debug))
         (force? (option-ref options 'force #f))
         (verbose? (or (command-line:get 'verbose)
                       (ide:command-line:get 'verbose)))
         (local-result EXIT_SUCCESS))

    (define (hello-handler message)
      (let ((hello (parse-message message)))
        (cond ((hello? hello)
               (let* ((data (hello-data hello))
                      (stdout (assoc-ref data "stdout"))
                      (connections (assoc-ref data "connections")))
                 (display stdout)
                 (when (or debug? verbose?)
                   (format #t
                           "connections: ~a\n"
                           (string-join connections)))))
              (else
               (format (current-error-port)
                       "ide hello: error: received: ~s\n"
                       message)
               (set! local-result EXIT_FAILURE)))
        message))

    (daemon:start #:debug? debug? #:force? force? #:port %ide-port)
    (let* ((commander (make-commander #:debug? debug? #:verbose? verbose?))
           (command (make-command "hello" '("dzn" "hello")))
           (result-handler
            (compose (make-finalizer commander #:debug? debug? #:verbose? verbose?)
                     hello-handler))
           (result
            (run-command commander command
                         #:result-handler result-handler
                         #:ide-port %ide-port #:debug? debug? #:verbose? verbose?)))
      (exit (+ result local-result)))))
